@@ -2,10 +2,10 @@ package Lemonldap::NG::Handler::Proxy;
 
 use strict;
 
-use Lemonldap::NG::Handler::Simple qw(:apache :headers);
+use Lemonldap::NG::Handler::Simple qw(:apache :headers :log);
 use LWP::UserAgent;
 
-our $VERSION = '0.3';
+our $VERSION = '0.31';
 
 ##########################################
 # COMPATIBILITY WITH APACHE AND APACHE 2 #
@@ -180,6 +180,16 @@ http://wiki.lemonldap.objectweb.org/xwiki/bin/view/NG/Presentation
 
 Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
+=head1 BUG REPORT
+
+Use OW2 system to report bug or ask for features:
+L<http://forge.objectweb.org/tracker/?group_id=274>
+
+=head1 DOWNLOAD
+
+Lemonldap::NG is available at
+L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2005-2007 by Xavier Guimard E<lt>x.guimard@free.frE<gt>
