@@ -4,12 +4,12 @@ use strict;
 
 use Lemonldap::NG::Handler::Simple qw(:all);
 use Lemonldap::NG::Handler::Vhost;
-use Lemonldap::NG::Manager;
+use Lemonldap::NG::Manager::Conf;
 use Cache::Cache qw($EXPIRES_NEVER);
 
 our @ISA = qw(Lemonldap::NG::Handler::Vhost Lemonldap::NG::Handler::Simple);
 
-our $VERSION    = '0.54';
+our $VERSION    = '0.61';
 our $cfgNum     = 0;
 our $lastReload = 0;
 our $reloadTime;
@@ -43,8 +43,22 @@ BEGIN {
 sub init($$) {
     my ( $class, $args ) = @_;
     $reloadTime = $args->{reloadTime} || 600;
-    $class->localInit($args);
     $localConfig = $args;
+    $class->localInit($args);
+}
+
+# defaultValuesInit : set default values for non-customized variables
+sub defaultValuesInit {
+    my ( $class, $args ) = @_;
+    # Local configuration overrides global configuration
+    $cookieName    = $localConfig->{cookieName} || $args->{cookieName} || 'lemonldap';
+    $cookieSecured = $localConfig->{cookieSecured} || $args->{cookieSecured} || 0;
+    $whatToTrace   = $localConfig->{whatToTrace} || $args->{whatToTrace}   || '$uid';
+    $whatToTrace =~ s/\$//g;
+    $https = $localConfig->{https} unless defined($https);
+    $https = $args->{https} unless defined($https);
+    $https = 1 unless defined($https);
+    1;
 }
 
 sub localInit {
@@ -122,7 +136,7 @@ sub globalConfUpdate {
     return $tmp unless ( ref($tmp) );
     # Local arguments have a best precedence
     foreach ( keys %$tmp ) {
-        $tmp->{$_} = $localConfig->{$_} if ( $localConfig->{$_} )
+        $tmp->{$_} = $localConfig->{$_} if ( $localConfig->{$_} );
     }
     $class->setConf($tmp);
     OK;
@@ -233,7 +247,7 @@ with the C<reloadTime> parameter (default 600s).
 
 =head3 getConf
 
-Call Lemonldap::NG::Manager::Conf qith the configStorage parameter.
+Call Lemonldap::NG::Manager::Conf with the configStorage parameter.
 
 =head1 OPERATION
 
@@ -253,8 +267,6 @@ local store.
 L<Lemonldap::NG::Handler>, L<Lemonldap::NG::Manager>, L<Lemonldap::NG::Portal>,
 http://wiki.lemonldap.objectweb.org/xwiki/bin/view/NG/Presentation
 
-=back
-
 =head1 AUTHOR
 
 Xavier Guimard, E<lt>x.guimard@free.frE<gt>
